@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/env python
 #
 # Urwid example similar to dialog(1) program
 #    Copyright (C) 2004-2009  Ian Ward
@@ -100,7 +100,7 @@ class DialogDisplay:
         self.loop = urwid.MainLoop(self.view, self.palette)
         try:
             self.loop.run()
-        except DialogExit, e:
+        except DialogExit as e:
             return self.on_exit( e.args[0] )
 
     def on_exit(self, exitcode):
@@ -230,12 +230,12 @@ class MenuItem(urwid.Text):
     def keypress(self,size,key):
         if key == "enter":
             self.state = True
-            raise DialogExit, 0
+            raise DialogExit(0)
         return key
     def mouse_event(self,size,event,button,col,row,focus):
         if event=='mouse release':
             self.state = True
-            raise DialogExit, 0
+            raise DialogExit(0)
         return False
     def get_state(self):
         return self.state
