@@ -40,7 +40,7 @@
 #endif
 
 static int widths_len = 2*38;
-static const long int widths[] = {
+static const int widths[] = {
     126, 1,
     159, 0,
     687, 1,
@@ -561,7 +561,7 @@ static Py_ssize_t Py_MovePrevChar(PyObject *text, Py_ssize_t start_offs,
     if (byte_encoding == ENC_UTF8) //encoding is utf8
     {
         position = end_offs - 1;
-        while ((str[position]&0xc0) == 0x80)
+        while ((position > start_offs) && (str[position]&0xc0) == 0x80)
             position -=1;
         return position;
     }
@@ -709,16 +709,16 @@ static PyObject * calc_width(PyObject *self, PyObject *args)
 {
     PyObject *text;
     int start_offs, end_offs;
-    int ret;
+    long ret;
 
     if (!PyArg_ParseTuple(args, "Oii", &text, &start_offs, &end_offs))
         return NULL; 
 
     ret = Py_CalcWidth(text, start_offs, end_offs);
-    if (ret==-1) //an error occured
+    if (ret==-1) //an error occurred
         return NULL;
-            
-    return Py_BuildValue("i", ret);
+
+    return Py_BuildValue("l", ret);
 }
 
 
@@ -843,7 +843,7 @@ static PyObject * calc_text_pos(PyObject *self, PyObject *args)
         return NULL;
 
     err = Py_CalcTextPos(text, start_offs, end_offs, pref_col, ret);
-    if (err==-1) //an error occured
+    if (err==-1) //an error occurred
         return NULL;
 
     return Py_BuildValue("(" FMT_N FMT_N ")", ret[0], ret[1]);
