@@ -18,6 +18,8 @@
 #
 # Urwid web site: http://excess.org/urwid/
 
+from __future__ import division, print_function
+
 import os
 import sys
 
@@ -26,12 +28,12 @@ try:
 except ImportError:
     pass # windows
 
-from urwid.util import int_scale
+from urwid.util import StoppingContext, int_scale
 from urwid import signals
-from urwid.compat import B, bytes3
+from urwid.compat import B, bytes3, xrange, with_metaclass
 
 # for replacing unprintable bytes with '?'
-UNPRINTABLE_TRANS_TABLE = B("?") * 32 + bytes3(range(32,256))
+UNPRINTABLE_TRANS_TABLE = B("?") * 32 + bytes3(list(xrange(32,256)))
 
 
 # signals sent by BaseScreen
@@ -67,11 +69,11 @@ _BASIC_COLOR_VALUES = [(0,0,0), (205, 0, 0), (0, 205, 0), (205, 205, 0),
     (0x5c, 0x5c, 0xff), (255, 0, 255), (0, 255, 255), (255, 255, 255)]
 
 _COLOR_VALUES_256 = (_BASIC_COLOR_VALUES +
-    [(r, g, b) for r in _CUBE_STEPS_256 for g in _CUBE_STEPS_256 
+    [(r, g, b) for r in _CUBE_STEPS_256 for g in _CUBE_STEPS_256
     for b in _CUBE_STEPS_256] +
     [(gr, gr, gr) for gr in _GRAY_STEPS_256])
 _COLOR_VALUES_88 = (_BASIC_COLOR_VALUES +
-    [(r, g, b) for r in _CUBE_STEPS_88 for g in _CUBE_STEPS_88 
+    [(r, g, b) for r in _CUBE_STEPS_88 for g in _CUBE_STEPS_88
     for b in _CUBE_STEPS_88] +
     [(gr, gr, gr) for gr in _GRAY_STEPS_88])
 
@@ -90,8 +92,10 @@ _STANDOUT = 0x02000000
 _UNDERLINE = 0x04000000
 _BOLD = 0x08000000
 _BLINK = 0x10000000
+_ITALICS = 0x20000000
+_STRIKETHROUGH = 0x40000000
 _FG_MASK = (_FG_COLOR_MASK | _FG_BASIC_COLOR | _FG_HIGH_COLOR |
-    _STANDOUT | _UNDERLINE | _BLINK | _BOLD)
+    _STANDOUT | _UNDERLINE | _BLINK | _BOLD | _ITALICS | _STRIKETHROUGH)
 _BG_MASK = _BG_COLOR_MASK | _BG_BASIC_COLOR | _BG_HIGH_COLOR
 
 DEFAULT = 'default'
@@ -133,19 +137,21 @@ _BASIC_COLORS = [
 
 _ATTRIBUTES = {
     'bold': _BOLD,
+    'italics': _ITALICS,
     'underline': _UNDERLINE,
     'blink': _BLINK,
     'standout': _STANDOUT,
+    'strikethrough': _STRIKETHROUGH,
 }
 
 def _value_lookup_table(values, size):
     """
     Generate a lookup table for finding the closest item in values.
     Lookup returns (index into values)+1
-    
+
     values -- list of values in ascending order, all < size
     size -- size of lookup table and maximum value
-    
+
     >>> _value_lookup_table([0, 7, 9], 10)
     [0, 0, 0, 0, 1, 1, 1, 1, 2, 2]
     """
@@ -181,11 +187,11 @@ _GRAY_88_LOOKUP_101 = [_GRAY_88_LOOKUP[int_scale(n, 101, 0x100)]
     for n in range(101)]
 
 
-# The functions _gray_num_256() and _gray_num_88() do not include the gray 
-# values from the color cube so that the gray steps are an even width.  
-# The color cube grays are available by using the rgb functions.  Pure 
-# white and black are taken from the color cube, since the gray range does 
-# not include them, and the basic colors are more likely to have been 
+# The functions _gray_num_256() and _gray_num_88() do not include the gray
+# values from the color cube so that the gray steps are an even width.
+# The color cube grays are available by using the rgb functions.  Pure
+# white and black are taken from the color cube, since the gray range does
+# not include them, and the basic colors are more likely to have been
 # customized by an end-user.
 
 
@@ -262,7 +268,7 @@ def _color_desc_88(num):
     0..15 -> 'h0'..'h15' basic colors (as high-colors)
     16..79 -> '#000'..'#fff' color cube colors
     80..87 -> 'g18'..'g90' grays
-    
+
     >>> _color_desc_88(15)
     'h15'
     >>> _color_desc_88(16)
@@ -295,7 +301,7 @@ def _parse_color_256(desc):
     '#000'..'#fff' -> 16..231 color cube colors
     'g0'..'g100' -> 16, 232..255, 231 grays and color cube black/white
     'g#00'..'g#ff' -> 16, 232...255, 231 gray and color cube black/white
-    
+
     Returns None if desc is invalid.
 
     >>> _parse_color_256('h142')
@@ -363,9 +369,9 @@ def _parse_color_88(desc):
     '#000'..'#fff' -> 16..79 color cube colors
     'g0'..'g100' -> 16, 80..87, 79 grays and color cube black/white
     'g#00'..'g#ff' -> 16, 80...87, 79 gray and color cube black/white
-    
+
     Returns None if desc is invalid.
-    
+
     >>> _parse_color_88('h142')
     >>> _parse_color_88('h42')
     42
@@ -438,7 +444,7 @@ class AttrSpec(object):
               'default' (use the terminal's default foreground),
               'black', 'dark red', 'dark green', 'brown', 'dark blue',
               'dark magenta', 'dark cyan', 'light gray', 'dark gray',
-              'light red', 'light green', 'yellow', 'light blue', 
+              'light red', 'light green', 'yellow', 'light blue',
               'light magenta', 'light cyan', 'white'
 
               High-color example values:
@@ -450,7 +456,8 @@ class AttrSpec(object):
               'h8' (color number 8), 'h255' (color number 255)
 
               Setting:
-              'bold', 'underline', 'blink', 'standout'
+              'bold', 'italics', 'underline', 'blink', 'standout',
+              'strikethrough'
 
               Some terminals use 'bold' for bright colors.  Most terminals
               ignore the 'blink' setting.  If the color is not given then
@@ -470,7 +477,7 @@ class AttrSpec(object):
 
         colors -- the maximum colors available for the specification
 
-                   Valid values include: 1, 16, 88 and 256.  High-color 
+                   Valid values include: 1, 16, 88 and 256.  High-color
                    values are only usable with 88 or 256 colors.  With
                    1 color only the foreground settings may be used.
 
@@ -498,12 +505,14 @@ class AttrSpec(object):
     foreground_number = property(lambda s: s._value & _FG_COLOR_MASK)
     background_basic = property(lambda s: s._value & _BG_BASIC_COLOR != 0)
     background_high = property(lambda s: s._value & _BG_HIGH_COLOR != 0)
-    background_number = property(lambda s: (s._value & _BG_COLOR_MASK) 
+    background_number = property(lambda s: (s._value & _BG_COLOR_MASK)
         >> _BG_SHIFT)
+    italics = property(lambda s: s._value & _ITALICS != 0)
     bold = property(lambda s: s._value & _BOLD != 0)
     underline = property(lambda s: s._value & _UNDERLINE != 0)
     blink = property(lambda s: s._value & _BLINK != 0)
     standout = property(lambda s: s._value & _STANDOUT != 0)
+    strikethrough = property(lambda s: s._value & _STRIKETHROUGH != 0)
 
     def _colors(self):
         """
@@ -543,8 +552,9 @@ class AttrSpec(object):
 
     def _foreground(self):
         return (self._foreground_color() +
-            ',bold' * self.bold + ',standout' * self.standout +
-            ',blink' * self.blink + ',underline' * self.underline)
+            ',bold' * self.bold + ',italics' * self.italics +
+            ',standout' * self.standout + ',blink' * self.blink +
+            ',underline' * self.underline + ',strikethrough' * self.strikethrough)
 
     def _set_foreground(self, foreground):
         color = None
@@ -556,7 +566,7 @@ class AttrSpec(object):
                 # parse and store "settings"/attributes in flags
                 if flags & _ATTRIBUTES[part]:
                     raise AttrSpecError(("Setting %s specified more than" +
-                        "once in foreground (%s)") % (repr(part), 
+                        "once in foreground (%s)") % (repr(part),
                         repr(foreground)))
                 flags |= _ATTRIBUTES[part]
                 continue
@@ -574,7 +584,7 @@ class AttrSpec(object):
                 flags |= _FG_HIGH_COLOR
             # _parse_color_*() return None for unrecognised colors
             if scolor is None:
-                raise AttrSpecError(("Unrecognised color specification %s" +
+                raise AttrSpecError(("Unrecognised color specification %s " +
                     "in foreground (%s)") % (repr(part), repr(foreground)))
             if color is not None:
                 raise AttrSpecError(("More than one color given for " +
@@ -595,7 +605,7 @@ class AttrSpec(object):
         if self._value & _HIGH_88_COLOR:
             return _color_desc_88(self.background_number)
         return _color_desc_256(self.background_number)
-        
+
     def _set_background(self, background):
         flags = 0
         if background in ('', 'default'):
@@ -621,7 +631,7 @@ class AttrSpec(object):
         Return (fg_red, fg_green, fg_blue, bg_red, bg_green, bg_blue) color
         components.  Each component is in the range 0-255.  Values are taken
         from the XTerm defaults and may not exactly match the user's terminal.
-        
+
         If the foreground or background is 'default' then all their compenents
         will be returned as None.
 
@@ -646,6 +656,13 @@ class AttrSpec(object):
         else:
             return vals + _COLOR_VALUES_256[self.background_number]
 
+    def __eq__(self, other):
+        return isinstance(other, AttrSpec) and self._value == other._value
+
+    def __ne__(self, other):
+        return not self == other
+
+    __hash__ = object.__hash__
 
 
 class RealTerminal(object):
@@ -653,8 +670,8 @@ class RealTerminal(object):
         super(RealTerminal,self).__init__()
         self._signal_keys_set = False
         self._old_signal_keys = None
-        
-    def tty_signal_keys(self, intr=None, quit=None, start=None, 
+
+    def tty_signal_keys(self, intr=None, quit=None, start=None,
         stop=None, susp=None, fileno=None):
         """
         Read and/or set the tty's signal character settings.
@@ -679,36 +696,35 @@ class RealTerminal(object):
         skeys = (sattr[termios.VINTR], sattr[termios.VQUIT],
             sattr[termios.VSTART], sattr[termios.VSTOP],
             sattr[termios.VSUSP])
-        
+
         if intr == 'undefined': intr = 0
         if quit == 'undefined': quit = 0
         if start == 'undefined': start = 0
         if stop == 'undefined': stop = 0
         if susp == 'undefined': susp = 0
-        
+
         if intr is not None: tattr[6][termios.VINTR] = intr
         if quit is not None: tattr[6][termios.VQUIT] = quit
         if start is not None: tattr[6][termios.VSTART] = start
         if stop is not None: tattr[6][termios.VSTOP] = stop
         if susp is not None: tattr[6][termios.VSUSP] = susp
-        
+
         if intr is not None or quit is not None or \
             start is not None or stop is not None or \
             susp is not None:
             termios.tcsetattr(fileno, termios.TCSADRAIN, tattr)
             self._signal_keys_set = True
-        
+
         return skeys
 
 
 class ScreenError(Exception):
     pass
 
-class BaseScreen(object):
+class BaseScreen(with_metaclass(signals.MetaSignals, object)):
     """
     Base class for Screen classes (raw_display.Screen, .. etc)
     """
-    __metaclass__ = signals.MetaSignals
     signals = [UPDATE_PALETTE_ENTRY, INPUT_DESCRIPTORS_CHANGED]
 
     def __init__(self):
@@ -718,12 +734,44 @@ class BaseScreen(object):
 
     started = property(lambda self: self._started)
 
-    def start(self):
+    def start(self, *args, **kwargs):
+        """Set up the screen.  If the screen has already been started, does
+        nothing.
+
+        May be used as a context manager, in which case :meth:`stop` will
+        automatically be called at the end of the block:
+
+            with screen.start():
+                ...
+
+        You shouldn't override this method in a subclass; instead, override
+        :meth:`_start`.
+        """
+        if not self._started:
+            self._start(*args, **kwargs)
         self._started = True
+        return StoppingContext(self)
+
+    def _start(self):
+        pass
 
     def stop(self):
+        if self._started:
+            self._stop()
         self._started = False
 
+    def _stop(self):
+        pass
+
+    def run_wrapper(self, fn, *args, **kwargs):
+        """Start the screen, call a function, then stop the screen.  Extra
+        arguments are passed to `start`.
+
+        Deprecated in favor of calling `start` as a context manager.
+        """
+        with self.start(*args, **kwargs):
+            return fn()
+
 
     def register_palette(self, palette):
         """Register a set of palette entries.
@@ -750,7 +798,7 @@ class BaseScreen(object):
                 raise ScreenError("Invalid register_palette entry: %s" %
                     repr(item))
             name, like_name = item
-            if not self._palette.has_key(like_name):
+            if like_name not in self._palette:
                 raise ScreenError("palette entry '%s' doesn't exist"%like_name)
             self._palette[name] = self._palette[like_name]
 
@@ -771,7 +819,7 @@ class BaseScreen(object):
             'light magenta', 'light cyan', 'white'
 
             Settings:
-            'bold', 'underline', 'blink', 'standout'
+            'bold', 'underline', 'blink', 'standout', 'strikethrough'
 
             Some terminals use 'bold' for bright colors.  Most terminals
             ignore the 'blink' setting.  If the color is not given then
