@@ -19,6 +19,8 @@
 #
 # Urwid web site: http://excess.org/urwid/
 
+from __future__ import division, print_function
+
 """
 HTML PRE-based UI implementation
 """
@@ -49,12 +51,12 @@ class HtmlGenerator(BaseScreen):
         self.colors = 16
         self.bright_is_bold = False # ignored
         self.has_underline = True # ignored
-        self.register_palette_entry(None, 
+        self.register_palette_entry(None,
             _default_foreground, _default_background)
 
     def set_terminal_properties(self, colors=None, bright_is_bold=None,
         has_underline=None):
-        
+
         if colors is None:
             colors = self.colors
         if bright_is_bold is None:
@@ -70,42 +72,36 @@ class HtmlGenerator(BaseScreen):
         """Not yet implemented"""
         pass
 
-    def start(self):
-        pass
-    
-    def stop(self):
-        pass
-    
     def set_input_timeouts(self, *args):
         pass
 
     def reset_default_terminal_palette(self, *args):
         pass
 
-    def run_wrapper(self,fn):
-        """Call fn."""
-        return fn()
-
-    def draw_screen(self, (cols, rows), r ):
-        """Create an html fragment from the render object. 
+    def draw_screen(self, size, r ):
+        """Create an html fragment from the render object.
         Append it to HtmlGenerator.fragments list.
         """
         # collect output in l
         l = []
-        
+
+        cols, rows = size
+
         assert r.rows() == rows
-    
+
         if r.cursor is not None:
             cx, cy = r.cursor
         else:
             cx = cy = None
-        
+
         y = -1
         for row in r.content():
             y += 1
             col = 0
-            
+
             for a, cs, run in row:
+                if not str is bytes:
+                    run = run.decode()
                 run = run.translate(_trans_table)
                 if isinstance(a, AttrSpec):
                     aspec = a
@@ -126,10 +122,10 @@ class HtmlGenerator(BaseScreen):
                     l.append(html_span(run, aspec))
 
             l.append("\n")
-                        
+
         # add the fragment to the list
         self.fragments.append( "<pre>%s</pre>" % "".join(l) )
-            
+
     def clear(self):
         """
         Force the screen to be completely repainted on the next
@@ -138,11 +134,11 @@ class HtmlGenerator(BaseScreen):
         (does nothing for html_fragment)
         """
         pass
-            
+
     def get_cols_rows(self):
         """Return the next screen size in HtmlGenerator.sizes."""
         if not self.sizes:
-            raise HtmlGeneratorSimulationError, "Ran out of screen sizes to return!"
+            raise HtmlGeneratorSimulationError("Ran out of screen sizes to return!")
         return self.sizes.pop(0)
 
     def get_input(self, raw_keys=False):
@@ -173,9 +169,9 @@ def html_span(s, aspec, cursor = -1):
     def html_span(fg, bg, s):
         if not s: return ""
         return ('<span style="color:%s;'
-            'background:%s%s">%s</span>' % 
+            'background:%s%s">%s</span>' %
             (fg, bg, extra, html_escape(s)))
-    
+
     if cursor >= 0:
         c_off, _ign = util.calc_text_pos(s, 0, len(s), cursor)
         c2_off = util.move_next_char(s, c_off, len(s))
@@ -195,17 +191,17 @@ def html_escape(text):
 
 def screenshot_init( sizes, keys ):
     """
-    Replace curses_display.Screen and raw_display.Screen class with 
+    Replace curses_display.Screen and raw_display.Screen class with
     HtmlGenerator.
-    
-    Call this function before executing an application that uses 
+
+    Call this function before executing an application that uses
     curses_display.Screen to have that code use HtmlGenerator instead.
-    
+
     sizes -- list of ( columns, rows ) tuples to be returned by each call
              to HtmlGenerator.get_cols_rows()
     keys -- list of lists of keys to be returned by each call to
             HtmlGenerator.get_input()
-    
+
     Lists of keys may include "window resize" to force the application to
     call get_cols_rows and read a new screen size.
 
@@ -227,21 +223,21 @@ def screenshot_init( sizes, keys ):
             assert type(row) == int
             assert row>0 and col>0
     except (AssertionError, ValueError):
-        raise Exception, "sizes must be in the form [ (col1,row1), (col2,row2), ...]"
-    
+        raise Exception("sizes must be in the form [ (col1,row1), (col2,row2), ...]")
+
     try:
         for l in keys:
             assert type(l) == list
             for k in l:
                 assert type(k) == str
     except (AssertionError, ValueError):
-        raise Exception, "keys must be in the form [ [keyA1, keyA2, ..], [keyB1, ..], ...]"
-    
-    import curses_display
+        raise Exception("keys must be in the form [ [keyA1, keyA2, ..], [keyB1, ..], ...]")
+
+    from . import curses_display
     curses_display.Screen = HtmlGenerator
-    import raw_display
+    from . import raw_display
     raw_display.Screen = HtmlGenerator
-    
+
     HtmlGenerator.sizes = sizes
     HtmlGenerator.keys = keys
 
@@ -252,4 +248,4 @@ def screenshot_collect():
     HtmlGenerator.fragments = []
     return l
 
-    
+
